//	*******************************************************************************
//	Name	:	extent.hxx
//	Author	:	I. Nakagawa
//	Create	:	2014/01/29
//	Modify	:	2017/03/10	2017/05/02
//	Modify	:	2018/05/23	add Vector2 ll , gl , ...
//	*******************************************************************************

#pragma once

#include	"vector.hxx"

template <typename T>
struct	Extent2	{
	Extent2		()															{}
//	Extent2		(const Vector2<T>& l, const Vector2<T>& g)	: L(l), G(g)	{}
	Extent2		(const Vector2<T>& v1,const Vector2<T>& v2)	{			L = v1.ExtL(v2) ;	G = v1.ExtG(v2) ;		}
	//---------------------------------------------------------------------------------------
	Extent2		Extent	(const Vector2<T> other)	const	{	return	Extent2(L.ExtL(other),	G.ExtG(other)) ;	}
	Extent2		Extent	(const Extent2	  other)	const	{	return	Extent2(L.ExtL(other.L),G.ExtG(other.G)) ;	}
	Vector2<T>	Volume	(void)						const	{	return	(G-L) ;		}
	Vector2<T>	Middle	(void)						const	{	return	(L+G)/2 ;	}
	//---------------------------------------------------------------------------------------
	Vector2<T>	ll		(void)						const	{	return	Vector2<T>(L.x,L.y) ;	}
	Vector2<T>	gl		(void)						const	{	return	Vector2<T>(G.x,L.y) ;	}
	Vector2<T>	lg		(void)						const	{	return	Vector2<T>(L.x,G.y) ;	}
	Vector2<T>	gg		(void)						const	{	return	Vector2<T>(G.x,G.y) ;	}
	Vector2<T>	ml		(void)						const	{	return	(ll()+gl())/2 ;		}
	Vector2<T>	mg		(void)						const	{	return	(lg()+gg())/2 ;		}
	Vector2<T>	lm		(void)						const	{	return	(ll()+lg())/2 ;		}
	Vector2<T>	gm		(void)						const	{	return	(gl()+gg())/2 ;		}
	Vector2<T>	mm		(void)						const	{	return	(Middle()) ;		}
	//---------------------------------------------------------------------------------------
	bool	operator==	(const Extent2& v)			const	{	return (L==v.L && G==v.G) ;				}
	bool	operator!=	(const Extent2& v)			const	{	return (L!=v.L || G!=v.G) ;				}
	bool		is_0	(void)						const	{	Extent2	e_0 ;	return	(*this==e_0) ;	}
	//---------------------------------------------------------------------------------------
	Vector2<T>	L ;
	Vector2<T>	G ;
	} ;

template <typename T>
struct	Extent3	{
	Extent3		()															{}
//	Extent3		(const Vector3<T>& l, const Vector3<T>& g)	: L(l), G(g)	{}
	Extent3		(const Vector3<T>& v1,const Vector3<T>& v2)	{			L = v1.ExtL(v2) ;	G = v1.ExtG(v2) ;		}
	//---------------------------------------------------------------------------------------
	Extent3		Extent	(const Vector3<T> other)	const	{	return	Extent3(L.ExtL(other),	G.ExtG(other)) ;	}
	Extent3		Extent	(const Extent3	  other)	const	{	return	Extent3(L.ExtL(other.L),G.ExtG(other.G)) ;	}
	Vector3<T>	Volume	(void)						const	{	return	(G-L) ;		}
	Vector3<T>	Middle	(void)						const	{	return	(L+G)/2 ;	}
	//---------------------------------------------------------------------------------------
	Vector3<T>	lll		(void)						const	{	return	Vector3<T>(L.x,L.y,L.z) ;	}
	Vector3<T>	gll		(void)						const	{	return	Vector3<T>(G.x,L.y,L.z) ;	}
	Vector3<T>	lgl		(void)						const	{	return	Vector3<T>(L.x,G.y,L.z) ;	}
	Vector3<T>	ggl		(void)						const	{	return	Vector3<T>(G.x,G.y,L.z) ;	}
	Vector3<T>	llg		(void)						const	{	return	Vector3<T>(L.x,L.y,G.z) ;	}
	Vector3<T>	glg		(void)						const	{	return	Vector3<T>(G.x,L.y,G.z) ;	}
	Vector3<T>	lgg		(void)						const	{	return	Vector3<T>(L.x,G.y,G.z) ;	}
	Vector3<T>	ggg		(void)						const	{	return	Vector3<T>(G.x,G.y,G.z) ;	}
	Vector3<T>	mll		(void)						const	{	return	(lll()+gll())/2 ;	}
	Vector3<T>	mgl		(void)						const	{	return	(lgl()+ggl())/2 ;	}
	Vector3<T>	mlg		(void)						const	{	return	(llg()+glg())/2 ;	}
	Vector3<T>	mgg		(void)						const	{	return	(lgg()+ggg())/2 ;	}
	Vector3<T>	lml		(void)						const	{	return	(lll()+lgl())/2 ;	}
	Vector3<T>	gml		(void)						const	{	return	(gll()+ggl())/2 ;	}
	Vector3<T>	lmg		(void)						const	{	return	(llg()+lgg())/2 ;	}
	Vector3<T>	gmg		(void)						const	{	return	(glg()+ggg())/2 ;	}
	Vector3<T>	llm		(void)						const	{	return	(lll()+llg())/2 ;	}
	Vector3<T>	glm		(void)						const	{	return	(gll()+glg())/2 ;	}
	Vector3<T>	lgm		(void)						const	{	return	(lgl()+lgg())/2 ;	}
	Vector3<T>	ggm		(void)						const	{	return	(ggl()+ggg())/2 ;	}
	Vector3<T>	lmm		(void)						const	{	return	(lll()+lgg())/2 ;	}
	Vector3<T>	gmm		(void)						const	{	return	(gll()+ggg())/2 ;	}
	Vector3<T>	mlm		(void)						const	{	return	(lll()+glg())/2 ;	}
	Vector3<T>	mgm		(void)						const	{	return	(lgl()+ggg())/2 ;	}
	Vector3<T>	mml		(void)						const	{	return	(lll()+ggl())/2 ;	}
	Vector3<T>	mmg		(void)						const	{	return	(llg()+ggg())/2 ;	}
	Vector3<T>	mmm		(void)						const	{	return	(Middle()) ;		}
	//---------------------------------------------------------------------------------------
	bool	operator==	(const Extent3& v)			const	{	return (L==v.L && G==v.G) ;				}
	bool	operator!=	(const Extent3& v)			const	{	return (L!=v.L || G!=v.G) ;				}
	bool		is_0	(void)						const	{	Extent3	e_0 ;	return	(*this==e_0) ;	}
	//---------------------------------------------------------------------------------------
	Vector3<T>	L ;
	Vector3<T>	G ;
	} ;

typedef		Extent2<double>		Ed2 ;
typedef		Extent3<double>		Ed3 ;

