//	*******************************************************************************
//	Name	:	Vector2.hxx
//	Author	:	I. Nakagawa
//	Create	:	2014/01/21
//	Modify	:	2014/09/02	VC 6
//	Modify	:	2014/09/08	Add operator!=
//	Modify	:	2016/03/18	float -> T
//	Modify	:	2017/03/08	Add operator* , *= (Vector2) , Ext
//	Modify	:	2018/05/25	Add Length(Vector2)
//	Modify	:	2018/06/21	change Dot , Cross
//	Modify	:	2018/08/30	Normalize T
//	*******************************************************************************

#pragma		once

#include	<cmath>
#include	"i_funct.hxx"

template <typename T>
struct	Vector2	{
//	Vector2	()										{}
	Vector2	(T v=0)		: x(v),	y(v)				{}
	Vector2	(T x, T y)	: x(x),	y(y)				{}

	//---------------------------------------------------------------------------------------
	Vector2	operator-	(void)				const	{	return Vector2(-x, -y);				}
	Vector2	operator+	(const Vector2& v)	const	{	return Vector2(x + v.x, y + v.y);	}
	Vector2	operator-	(const Vector2& v)	const	{	return Vector2(x - v.x, y - v.y);	}
	Vector2	operator*	(const T s)			const	{	return Vector2(x * s, y * s);		}
	Vector2	operator/	(const T s)			const	{	return Vector2(x / s, y / s);		}
	Vector2	operator*	(const Vector2& v)	const	{	return Vector2(x * v.x, y * v.y);	}
	void	operator+=	(const Vector2& v)			{	*this = Vector2(x + v.x, y + v.y);	}
	void	operator-=	(const Vector2& v)			{	*this = Vector2(x - v.x, y - v.y);	}
	void	operator*=	(const T s)					{	*this = Vector2(x * s, y * s);		}
	void	operator/=	(const T s)					{	*this = Vector2(x / s, y / s);		}
	void	operator*=	(const Vector2& v)			{	*this = Vector2(x * v.x, y * v.y);	}
	//---------------------------------------------------------------------------------------
	bool	operator==	(const Vector2& v)	const	{	return x == v.x && y == v.y;		}
	bool	operator!=	(const Vector2& v)	const	{	return x != v.x || y != v.y;		}
	//---------------------------------------------------------------------------------------
	Vector2	ExtL		(const Vector2& v)	const	{	return	Vector2(i_min(x,v.x),i_min(y,v.y)) ;	}
	Vector2	ExtG		(const Vector2& v)	const	{	return	Vector2(i_max(x,v.x),i_max(y,v.y)) ;	}
	//---------------------------------------------------------------------------------------
	T	LengthSquared	(void)				const	{	return	(x * x + y * y) ;			}
	#if		_MSC_VER	<	1300
	double	Length		(void)				const	{	return	(   ::sqrt(LengthSquared())) ;		}
	#else
	double	Length		(void)				const	{	return	(std::sqrt(LengthSquared())) ;		}
	#endif
	double	Length		(const Vector2& v)	const	{	return	(v-*this).Length();					}
	void	Normalize	(void)						{
														if (Length() > 0) {
															double	s = 1.0f / Length() ;
															x = T(x*s) ;
															y = T(y*s) ;

															}
														}
	Vector2	Normalized	(void)				const	{
														Vector2 v = *this;
														v.Normalize();
														return v;
														}
	operator Vector2<float>	()				const	{	return Vector2<float>(x, y);		}
	//---------------------------------------------------------------------------------------
	double	Dot			(const Vector2& v)	const	{	return		 x * v.x + y * v.y ;	}
	double	Cross		(const Vector2& v)	const	{	return  	(

																		x * v.y - y * v.x);
														}
	const T* Pointer	(void)				const	{	return &x ;							}
	Vector2	Lerp(float t, const Vector2& v)	const	{	return Vector2(	x * (1 - t) + v.x * t,
																		y * (1 - t) + v.y * t);

														}
	template <typename P>
		P* Write(P* pData)							{
														Vector2* pVector = (Vector2*) pData;
														*pVector++ = *this;
														return (P*) pVector;
														}
	//---------------------------------------------------------------------------------------
	T	x ;
	T	y ;
	} ;

//	T		Dot			(const Vector2& v)	const	{	return		 x * v.x + y * v.y ;	}
//	double	Dot			(const Vector2& v)	const	{	double	dp = x * v.x + y * v.y ;
//														if (dp < -1.)	{	dp = -1. ;		}
//														if (dp >  1.)	{	dp =  1. ;		}
//														return dp ;							}
//	double	Cross		(const Vector2& v)	const	{	double	cp =	x * v.y - y * v.x ;
//														if (cp < -1.)	{	cp = -1. ;		}
//														if (cp >  1.)	{	cp =  1. ;		}
//														return cp ;							}
