//	*******************************************************************************
//	Name	:	Vector3.hxx
//	Author	:	I. Nakagawa
//	Create	:	2014/01/21
//	Modify	:	2014/09/02	VC 6
//	Modify	:	2014/09/08	Add operator!=
//	Modify	:	2015/06/24	Bug operator!=
//	Modify	:	2017/03/08	Add operator* , *= (Vector3) , Ext
//	Modify	:	2018/05/25	Add Length(Vector3)
//	Modify	:	2018/06/21	change Dot , Cross
//	Modify	:	2018/08/30	Normalize T
//	*******************************************************************************

#pragma		once

#include	<cmath>
#include	"i_funct.hxx"

template <typename T>
struct	Vector3 {
//	Vector3	()													{}
	Vector3	(T v=0)					:	x(v),	y(v),	z(v)	{}
	Vector3	(T x, T y, T z)			:	x(x),	y(y),	z(z)	{}
	Vector3	(const Vector2<T>& v)	:	x(v.x),	y(v.y),	z(0)	{}
	//---------------------------------------------------------------------------------------
	Vector3 operator-	(void)				const	{	return Vector3(-x, -y, -z);					}
	Vector3	operator+	(const Vector3& v)	const	{	return Vector3(x + v.x, y + v.y,  z + v.z);	}
	Vector3	operator-	(const Vector3& v)	const	{	return Vector3(x - v.x, y - v.y,  z - v.z);	}
	Vector3	operator*	(const T s)			const	{	return Vector3(x * s, y * s, z * s);		}
	Vector3	operator/	(const T s)			const	{	return Vector3(x / s, y / s, z / s);		}
	Vector3	operator*	(const Vector3& v)	const	{	return Vector3(x * v.x, y * v.y,  z * v.z);	}
	void	operator+=	(const Vector3& v)			{	x += v.x;	y += v.y;	z += v.z;			}
	void	operator-=	(const Vector3& v)			{	x -= v.x;	y -= v.y;	z -= v.z;			}
	void	operator*=	(const T s)					{	x *= s ;	y *= s;		z *= s ;			}
	void	operator/=	(const T s)					{	x /= s;		y /= s;		z /= s;				}
	void	operator*=	(const Vector3& v)			{	x *= v.x;	y *= v.y;	z *= v.z;			}
	//---------------------------------------------------------------------------------------
	bool	operator==	(const Vector3& v)	const	{	return x == v.x && y == v.y && z == v.z;	}
	bool	operator!=	(const Vector3& v)	const	{	return x != v.x || y != v.y || z != v.z;	}
	//---------------------------------------------------------------------------------------
	Vector3	ExtL		(const Vector3& v)	const	{	return	Vector3(i_min(x,v.x),i_min(y,v.y),i_min(z,v.z)) ;	}
	Vector3	ExtG		(const Vector3& v)	const	{	return	Vector3(i_max(x,v.x),i_max(y,v.y),i_max(z,v.z)) ;	}
	//---------------------------------------------------------------------------------------
	T	LengthSquared	(void)				const	{	return	(x * x + y * y + z * z) ;			}
	#if		_MSC_VER	<	1300
	double	Length		(void)				const	{	return	(   ::sqrt(LengthSquared())) ;		}
	#else
	double	Length		(void)				const	{	return	(std::sqrt(LengthSquared())) ;		}
	#endif
	double	Length		(const Vector3& v)	const	{	return	(v-*this).Length();					}
	void	Normalize	(void)						{
														if (Length() > 0) {
															double	s = 1.0f / Length() ;
															x = T(x*s) ;
															y = T(y*s) ;
															z = T(z*s) ;
															}
														}
	Vector3	Normalized	(void)				const	{
														Vector3 v = *this;
														v.Normalize();
														return v;
														}

	//---------------------------------------------------------------------------------------
	double	Dot			(const Vector3& v)	const	{	return		 x * v.x + y * v.y + z * v.z ;	}
	Vector3	Cross		(const Vector3& v)	const	{	return Vector3(	y * v.z - z * v.y,
																		z * v.x - x * v.z,
																		x * v.y - y * v.x);
														}
	const T* Pointer	(void)				const	{	return &x ;							}
	Vector3 Lerp(float t, const Vector3& v)	const	{	return Vector3(	x * (1 - t) + v.x * t,
																		y * (1 - t) + v.y * t,
																		z * (1 - t) + v.z * t);
														}
	template <typename P>
		P* Write(P* pData)							{
														Vector3<T>* pVector = (Vector3<T>*) pData;
														*pVector++ = *this;
														return (P*) pVector;
														}
	//---------------------------------------------------------------------------------------
	T	x ;
	T	y ;
	T	z ;
	} ;

//	T		Dot			(const Vector3& v)	const	{	return		 x * v.x + y * v.y + z * v.z ;	}
//	double	Dot			(const Vector3& v)	const	{	double	dp = x * v.x + y * v.y + z * v.z ;
//														if (dp < -1.)	{	dp = -1. ;		}
//														if (dp >  1.)	{	dp =  1. ;		}
//														return dp ;							}
