//	*******************************************************************************
//	Name	:	Vector4.hxx
//	Author	:	I. Nakagawa
//	Create	:	2014/01/21
//	Modify	:	2014/09/08	Add operator!=
//	Modify	:	2018/03/01	Add operator-
//	*******************************************************************************

#pragma once
#include <cmath>

template <typename T>
struct	Vector4	{
//	Vector4	()																{}
	Vector4	(T v=0)						:	x(v),	y(v),	z(v),	w(v)	{}
	Vector4	(T x,T y,T z,		 T w)	:	x(x),	y(y),	z(z),	w(w)	{}
	Vector4	(const Vector3<T>& v,T w)	:	x(v.x),	y(v.y),	z(v.z),	w(w)	{}
	//---------------------------------------------------------------------------------------
	Vector4	operator-	(void)				const	{	return Vector4(-x, -y, -z, -w);							}
	Vector4	operator+	(const Vector4& v)	const	{	return Vector4(x + v.x, y + v.y,  z + v.z, w + v.w);	}
	Vector4	operator-	(const Vector4& v)	const	{	return Vector4(x - v.x, y - v.y,  z - v.z, w - v.w);	}
	Vector4	operator*	(T s)				const	{	return Vector4(x * s, y * s, z * s, w * s);				}
	Vector4	operator/	(T s)				const	{	return Vector4(x / s, y / s, z / s, w / s);				}
	//---------------------------------------------------------------------------------------
	bool	operator==	(const Vector4& v)	const	{	return x == v.x && y == v.y && z == v.z && w == v.w ;	}
	bool	operator!=	(const Vector4& v)	const	{	return x != v.x || y != v.y || z != v.z || w != v.w ;	}
	//---------------------------------------------------------------------------------------
	T		Dot			(const Vector4& v)	const	{	return x * v.x + y * v.y + z * v.z + w * v.w;			}
	Vector4	Lerp(float t, const Vector4& v)	const	{	return Vector4(	x * (1 - t) + v.x * t,
																		y * (1 - t) + v.y * t,
																		z * (1 - t) + v.z * t,
																		w * (1 - t) + v.w * t);
														}
	const T* Pointer	(void)				const	{	return &x;												}
	//---------------------------------------------------------------------------------------
	T	x ;
	T	y ;
	T	z ;
	T	w ;
	} ;
