//	*******************************************************************************
//	Name	:	vxx_fnc3.hpp
//			:	Vxx	Functions
//	Author	:	I. Nakagawa
//	Create	:	2014/09/02
//	Modify	:	2014/09/02
//	*******************************************************************************

#pragma		once

#include	"i_raddeg.hxx"
#include	"vector.hxx"

////
//	*******************************************************************************
//	spin
//	Create	:	2014/09/02
//	*******************************************************************************
inline	Vd2	Vd2_Spin		(Vd2& other,const double theta)
{
	Vd2	this_ ;
	Vd2	spin ;
	double	sinT = sin(theta) ;
	double	cosT = cos(theta) ;
//	ReviseSinCos(theta,&sinT,&cosT) ;
	spin.x = (other.x-this_.x) * cosT - (other.y-this_.y) * sinT + this_.x ;
	spin.y = (other.x-this_.x) * sinT + (other.y-this_.y) * cosT + this_.y ;
	return	spin ;
	}

////
//	*******************************************************************************
//	angle
//	Create	:	2014/09/02
//	*******************************************************************************
inline	double	Vd2_Angle(const Vd2& p1,const Vd2& p2)
{
	if (p1 == p2)			{	return	0. ;				}
	Vd2	o ;
	Vd2	oa = p1 - o ;
	Vd2	ob = p2 - o ;
	double	xya = oa.Length() ;
	double	xyb = ob.Length() ;
	double	ta ,	tb ;
	if (xya == 0.)			{	ta = 0. ;					}
	else					{	ta = asin(oa.y/xya) ;		}
	if (oa.x < 0.)			{	ta = PI - ta ;				}
	if (xyb == 0.)			{	tb = 0. ;					}
	else					{	tb = asin(ob.y/xyb) ;		}
	if (ob.x < 0.)			{	tb = PI - tb ;				}
	if ((tb-ta) < rad(0.))	{	return	tb-ta+rad(360.) ;	}
	else					{	return	tb-ta ;				}
	}

////
//	*******************************************************************************
//	dot
//	Create	:	2014/09/02
//	*******************************************************************************
inline	double	Vd2_Dot	(const Vd2& p1,const Vd2& p2)
{
	Vd2	v1 = p1.Normalized() ;
	Vd2	v2 = p2.Normalized() ;
	return	v1.Dot(v2) ;
	}

////
//	*******************************************************************************
//	angle
//	Create	:	2014/09/02
//	*******************************************************************************
inline	double	Eye_GetAngleXY	(const Vd3& eye)
{
	Vd2	eyeXY = Vd2(eye.x,eye.y) ;
	Vd2	eye0 ;
//	return	::Vd2_Angle	(Vd2(1,0),eyeXY) ;
	return	acos(Vd2_Dot(Vd2(1,0),eyeXY)) ;
	}

inline	double	Eye_GetAngleZ	(const Vd3& eye)
{
	Vd2	eyeYZ = Vd2(fabs(eye.y),eye.z) ;
	Vd2	eye0 ;
//	double	angleZ = ::Vd2_Angle (Vd2(1,0),eyeYZ) ;
	double	angleZ = acos(Vd2_Dot(Vd2(1,0),eyeYZ)) ;
	if (angleZ > rad(180))	{	angleZ -= rad(360) ;	}
	return	angleZ ;
	}

////
//	*******************************************************************************
//	spin
//	Create	:	2014/09/02
//	*******************************************************************************
inline	Vd3	Eye_GetSpinXY	(const Vd3& eye,const double theta)
{
	Vd2	exy = Vd2(eye.x,eye.y) ;
//	Vd2	txy = Vd2(0) ;
	Vd2	nxy = ::Vd2_Spin(exy,theta) ;
	Vd3	nEye(nxy.x,nxy.y,eye.z) ;
	return	nEye ;
	}

inline	Vd3	Eye_GetSpin_Z	(const Vd3& eye,const double theta)
{
	double	newAngle = ::Eye_GetAngleZ(eye)+theta ;
	if		(rad(   0) <= newAngle && newAngle<rad(  89))		{							}
	else if	(rad(  89) <= newAngle && newAngle<rad( 180))		{	newAngle = rad( 89) ;	}
	else if (rad( 180) <= newAngle && newAngle<rad( 271))		{	newAngle = rad(-89) ;	}
	else if (rad( 271) <= newAngle && newAngle<rad( 360))		{							}
	else if	(rad( -89) <= newAngle && newAngle<rad(   0))		{							}
	else if	(rad(-180) <= newAngle && newAngle<rad( -89))		{	newAngle = rad(-89) ;	}
	else if (rad(-271) <= newAngle && newAngle<rad(-180))		{	newAngle = rad( 89) ;	}
	else if (rad(-360) <= newAngle && newAngle<rad(-271))		{							}
	double	newTheta = newAngle-::Eye_GetAngleZ(eye) ;
	if (eye.y<0) {	newTheta = -newTheta ;	}
	Vd2		eyz = Vd2(eye.y,eye.z) ;
//	Vd2		tyz ;
	Vd2		nyz = ::Vd2_Spin(eyz,newTheta) ;
	Vd3		nEye(eye.x,nyz.x,nyz.y) ;
	return	nEye ;
	}

